@@ -23,8 +23,8 @@ int main()
 	if (!m) { msg(F_MISC, L_ERR,"main: ERROR: Couldn't open dictionary!\n");
 		  return 1; }
 
-	spc = meta_getspcbynr(m, C_DS_RAD_PKT);
-	if (!spc) { msg(F_MISC, L_ERR, "main: ERROR: RAD-PKT not defined!\n");
+	spc = meta_getspcbynr(m, C_DS_GROUND);
+	if (!spc) { msg(F_MISC, L_ERR, "main: ERROR: Ground space not defined!\n");
 		    return 1; }
 
 	/* Get ASCII pairs from stdin into META_AV list */
