@@ -25,6 +25,7 @@
  * 		    - Fixed small ! operator behaviour bug on strings
  * 2003/12/30 - EvB - Added EXISTS instruction that tests for a resolved
  * 		      attribute reference
+ * 2004/08/14 - EvB - Added STR2MAC instruction to canonicalize MAC addresses
  */
 
 char langvm[] = "LANGVM - Copyright (C) 2001 Emile van Bergen.";
@@ -39,6 +40,7 @@ char langvm[] = "LANGVM - Copyright (C) 2001 Emile van Bergen.";
 #include <string.h>		/* For memset() */
 #include <unistd.h>		/* For write() in printav */
 #include <time.h>		/* For time_t, gmtime() and strftime() */
+#include <ctype.h>		/* For touppper(), tolower() */
 
 #include <md5.h>
 #include <misc.h>
@@ -128,6 +130,31 @@ META_ORD op_datstr2ord(char *p, META_ORD l)
 	return mktime(&tm);
 }
 
+/* Canonicalize MAC addresses, using custom output delimiter. Assumes there's
+   enough room in out to store 12 hex digits + 5 times the output delimiter */
+
+int canmac(char *out, char *in, META_ORD inl, char *od, META_ORD odl)
+{
+	int n, rl;
+	char *i, *o, c;
+
+	i = in; o = out; 
+	for(n = 0; ; n++) {
+		c = 0;
+		rl = (in + inl) - i;
+		if (rl > 0) {
+			c = meta_atoord(i, MIN(rl, 2), 0, 0, &rl, 16); 
+			i += rl;
+			if (i < in + inl && !ISHEX(*i)) i++;
+		}
+		hex(o, &c, 1); o += 2;
+		if (n == 5) break;
+		if (!odl) continue;
+		memcpy(o, od, odl); o += odl;
+	}
+	return o - out;
+}
+
 
 /* Utility macros for use inside vm_run */
 
@@ -332,6 +359,20 @@ int vm_run(VM *vm)
 			hex(s, R.p, R.l);
 			CHGSTR(&R, s, R.l << 1);
 			break; 
+		  case OP_LOWER:
+		  	if (!R.l) break;
+			if (R.flags & AV_FREE_P) s = R.p;
+			else { NEWSTR(s, R.l); R.flags |= AV_FREE_P; }
+			for(n = 0; n < R.l; n++) s[n] = tolower(R.p[n]);
+			R.p = s;
+			break;
+		  case OP_UPPER:
+		  	if (!R.l) break;
+			if (R.flags & AV_FREE_P) s = R.p;
+			else { NEWSTR(s, R.l); R.flags |= AV_FREE_P; }
+			for(n = 0; n < R.l; n++) s[n] = toupper(R.p[n]);
+			R.p = s;
+			break;
 
 		  /*
 		   * Binary string operators
@@ -547,6 +588,11 @@ int vm_run(VM *vm)
 			CHGSTR(&L, s, strlen(s));
 			L.i = 0;
 			break;
+		  case OP_STR2MAC:
+			NEWSTR(s, 13 + R.l * 5);
+			n = canmac(s, L.p, L.l, R.p, R.l);
+			CHGSTR(&L, s, n);
+			break; 
 
 		  /*
 		   * Assignment etc.
@@ -620,6 +666,18 @@ int vm_run(VM *vm)
 			L.flags &= ~(AV_UNRESOLVED|AV_FREE_P);
 			break;
 
+		  case OP_POKEAV:
+			/* Return an anonymous zero if we don't succeed */
+			if (!L.sub || !L.sub->p || !L.sub->l || !R.p || !R.l) {
+				meta_freeavdata(&L); 
+				L.sub = 0; L.i = 0; 
+				L.p = 0; L.l = 0; 
+				L.flags |= AV_UNRESOLVED;
+				break;
+			}
+			memcpy(L.sub->p, R.p, MIN(L.sub->l, R.l));
+			break;
+
 		  case OP_DELAV:
 		  	if (!R.sub) { 			/* If none, return 0. */
 				meta_freeavdata(&R);	/* In case of del 'a' */
