@@ -345,9 +345,9 @@ U_INT32_T do_search(LDAP *ld, char *basedn, char *filter,
 	static char dbgbuf[MAPF_LINELEN];
 	struct timeval tv;
 	LDAPMessage *res, *ent;
-	struct berval **vals, **v;
+	struct berval **vals, **v, bv;
 	U_INT32_T *o;
-	char *s;
+	char *s, bv_buf[1024];
 	int n, m, cnt;
 
 	o = msgbuf + 2; cnt = 0;
@@ -378,10 +378,49 @@ U_INT32_T do_search(LDAP *ld, char *basedn, char *filter,
 	    for(n = 0; n < map->entries; n++) {
 
 		vals = ldap_get_values_len(ld, ent, map->ldap_atrs[n]);
-		if (!vals) continue;
 
 		if (debug) fprintf(stderr, "         Object's values for %s:\n",
 				   map->ldap_atrs[n]);
+		if (!vals) {
+		    if (strcasecmp(map->ldap_atrs[n], "dn") == 0) {
+			/* DN holen und kopieren */
+			s = ldap_get_dn(ld, ent);
+			bv.bv_len = strlen(s);
+			bv.bv_val = bv_buf;
+			strcpy(bv_buf, s);
+			ldap_memfree(s);
+
+			if (debug) {
+				m = bv.bv_len; 
+				if (m > MAPF_LINELEN - 1) m = MAPF_LINELEN - 1;
+				memcpy(dbgbuf, bv.bv_val, m); dbgbuf[m] = 0;
+				fprintf(stderr, "             %s (len %ld)\n", 
+					dbgbuf, bv.bv_len);
+			}
+			
+			/* Check if there's room, considering current
+			 * and new size */
+			m = bv.bv_len;
+			if (((o - msgbuf + 
+			      4 + ((m + 3) >> 2)) << 2) >= C_MAX_MSGSIZE) {
+
+				fprintf(stderr, "radldap: No more room for "
+						"attribute '%s', len %d!\n",
+					map->ldap_atrs[n], m);
+				break;
+			}
+
+			/* Write attribute to message buffer and advance ptr */
+			o[0] = htonl(map->spcs[n]);
+			o[1] = htonl(map->vnds[n]);
+			o[2] = htonl(map->atrs[n]);
+			o[3] = htonl(m);
+			memcpy(&(o[4]), bv.bv_val, m);
+			o += 4 + ((m + 3) >> 2);
+		    }
+		    
+		    continue;
+		}
 
 		for(v = vals; *v; v++) {
 			if (debug) {
