@@ -39,9 +39,7 @@ char bigtest_id[] = "BIGTEST - Copyright (C) 2001 Emile van Bergen.";
 #include <jobs.h>		/* Also includes srvtypes.h */
 #include <channels.h>		/* For chan_handle_* */
 
-#ifndef RADDB
-#define RADDB		"../raddb"
-#endif
+#include <defs.h>
 
 
 /*
@@ -127,7 +125,7 @@ int main()
 		if (sigchld) {
 
 			/* Handle exited childs, if any */
-			while((n = wait4(-1, &status, WNOHANG, 0)) > 0) {
+			while((n = waitpid(0, &status, WNOHANG)) > 0) {
 
 				/* Find out which of our subprocesses exited */
 				for(p = c->procs; p && p->pid != n; p=p->next);
